# Microeconometrics
# CLASS 9a
# Panel data for continous variable
# Wiktor Budzinski

rm(list=ls())
library(plm)
library(readxl)
library(stargazer)
library(sandwich)
library(lmtest)
library(nlme)
library(zoo)

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 9")         # sets the access path (sometimes change the characters '\' to '/' is needed)

############################ Exercise 1 ##################################

Wages <- read_excel("panelwages.xlsx")

# exp             float   %9.0g                 years of full-time work experience
# wks             float   %9.0g                 weeks worked
# occ             float   %9.0g                 occupation; occ==1 if in a blue-collar occupation
# ind             float   %9.0g                 industry; ind==1 if working in a manufacturing industry
# south           float   %9.0g                 residence; south==1 if in the South area
# smsa            float   %9.0g                 smsa==1 if in the Standard metropolitan statistical area
# ms              float   %9.0g                 marital status
# fem             float   %9.0g                 female or male
# union           float   %9.0g                 if wage set be a union contract
# ed              float   %9.0g                 years of education
# blk             float   %9.0g                 black
# lwage           float   %9.0g                 log wage
# id              float   %9.0g                 
# t               float   %9.0g                 
# tdum1           byte    %8.0g                 t== 1.0000
# tdum2           byte    %8.0g                 t== 2.0000
# tdum3           byte    %8.0g                 t== 3.0000
# tdum4           byte    %8.0g                 t== 4.0000
# tdum5           byte    %8.0g                 t== 5.0000
# tdum6           byte    %8.0g                 t== 6.0000
# tdum7           byte    %8.0g                 t== 7.0000
# exp2            float   %9.0g                 

Eq <- lwage ~ exp + exp2 + wks + ed +  south + smsa + blk + occ + ind
m<-lm(Eq, data = Wages)
summary(m)
coeftest(m, vcov = vcovCL(m, "HC1", cluster = Wages$id)) # Clustered SE

# Defining panel data
Panel_data<-pdata.frame(Wages,index = c("id","t"))

m_b <- plm(Eq, data = Panel_data, model = "between")
summary(m_b)
m_f <- plm(Eq, data = Panel_data, model = "within")
summary(m_f)
coeftest(m_f, vcovBK(m_f, "HC1", cluster = "group"))

# Theta says how much of the overall variation is explained by the individual variation
m_r <- plm(Eq, data = Panel_data, model = "random")
summary(m_r)
coeftest(m_r, vcovBK(m_r, "HC1", cluster = "group")) 
stargazer(m_b, m_f, m_r, type="text", df=FALSE)  

# Are the individual effects significant?
pFtest(Eq, Panel_data,effect = "individual") # F-test
# Breush-pagan test 
m_p <- plm(Eq, data = Panel_data, model = "pooling") # Pooled model
plmtest(m_p)

# Is fixed effect more appropriate?
  # Are error components correlated with Xs?
phtest(m_f, m_r) # Hausman test 

# Angrist and Newey test
aneweytest(Eq, Panel_data, index = "id")

# Twoway effects model
Eq2 <- lwage ~ exp + exp2 + wks + ed +  south + smsa + blk + occ + ind + factor(t)
m_ft <- plm(Eq2, data = Panel_data, model = "within")
summary(m_ft)

Eq2 <- lwage ~ exp + exp2 + wks + ed +  south + smsa + blk + occ + ind
m_ft2 <- plm(Eq2, data = Panel_data, model = "within", effect = "twoway")
summary(m_ft2)

# Breush-pagan test for twoway effects
plmtest(m_p, effect = "twoways")


########## Serial autocorrelation
# Wooldridge test for serial autocorrelation [AR(1)] (within model)
pwartest(Eq, data = Panel_data)

## Serial correlation [AR(1)] in random effects models
pbsytest(Eq, data = Panel_data, test = "J") # Joint
pbsytest(Eq, data = Panel_data, test = "RE") # RE (time invariant effect)
pbsytest(Eq, data = Panel_data, test = "AR") # serial correlation

pbltest(Eq, data = Panel_data, alternative = "onesided")

m_r2 <- lme(Eq, data = Wages, random = ~1 | id) #RE model estimated with MLE
summary(m_r2)

# AR(1) model
m_r_ar1 <- lme(Eq, data = Wages, random = ~1 | id,
              correlation = corAR1(0, form = ~ t | id))
summary(m_r_ar1)
anova(m_r2, m_r_ar1) # Likelihood ratio test

############################ Exercise 2 ##################################
# DiD - Napster

rm(list=ls())
Napster <- read_excel("napster.xlsx")

hist(Napster$cdall)
summary(Napster$cdall)

Napster$Date <- as.yearmon(paste(Napster$year, Napster$firmth), "%Y %m")
Napster$napster<-Napster$year >= 2000 | (Napster$year == 1999 & Napster$firmth >=6)
Napster$intaccess <- (1-Napster$nint)
Napster$treat <- Napster$napster*(1-Napster$nint)

# Getting rid of individuals who only answered the survey once
B<-unique(Napster$newid)
Napster$NoObs <- Napster$age
for(i in 1:length(B)){
  Napster$NoObs[Napster$newid == B[i]] <- sum(Napster$newid == B[i])
}
Napster <- Napster[Napster$NoObs > 1,]

Panel_data <- pdata.frame(Napster,index = c("newid","Date"))

Eq <- cdall ~ treat + intaccess + napster + age + white + black + male + hsgrad + lesscol + colgrad + fam_size + 
  single +retired + spouwrk+  fincreal + owner + renter + msa + tv + comp + sound + vcr +vehq

m_p <- plm(Eq, data = Panel_data, model = "pooling") # Pooled model
summary(m_p)

m_f <- plm(Eq, data = Panel_data, model = "within")
summary(m_f)
coeftest(m_f, vcovBK(m_f, "HC1", cluster = "group"))

m_r <- plm(Eq, data = Panel_data, model = "random")
summary(m_r)
phtest(m_f, m_r) # Hausman test 

pFtest(Eq, Panel_data,effect = "individual") # F-test
plmtest(m_p)
pwartest(Eq, data = Panel_data)

# Time effects

Eq2 <- cdall ~ treat + intaccess + napster + factor(year) + age + white + black + male + hsgrad + lesscol + colgrad + fam_size + 
  single +retired + spouwrk+  fincreal + owner + renter + msa + tv + comp + sound + vcr +vehq
m_ft <- plm(Eq2, data = Panel_data, model = "within")
summary(m_ft)
coeftest(m_ft, vcovBK(m_ft, "HC1", cluster = "group"))