# Microeconometrics
# CLASS 9c
# Random parameters in choice models
# Wiktor Budzinski

rm(list=ls())
setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 9")         # sets the access path (sometimes change the characters '\' to '/' is needed)

# install.packages("tidyr") # Install packages if necessary 
library(apollo)
library(ramify)
library(tidyr)
library(mvtnorm)
# Output settings
modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE
set.seed(100000001) 

# Read data
load("TPN.RData")

database$Passive_1 <- database$Passive_1/10
database$Passive_2 <- database$Passive_2/10
database$Passive_3 <- database$Passive_3/10

database$Cost_1 <- database$Cost_1/10
database$Cost_2 <- database$Cost_2/10
database$Cost_3 <- database$Cost_3/10
##############################################################################################
####                                          MNL                                         ####
##############################################################################################

apollo_initialise()
apollo_control = list(modelName  ="MNL",modelDescr ="MNL for TPN",
                      indivID    ="ID", nCores = 4, outputDirectory = "output")
# SPECIFY COEFFICIENTS
apollo_beta = c(b_sq = 0.1, b_pass = 0.1, b_cost = 0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = b_pass*Passive_1 + b_cost*Cost_1
  V[['alt2']]  = b_pass*Passive_2 + b_cost*Cost_2
  V[['alt3']]  = b_sq + b_pass*Passive_3 + b_cost*Cost_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = 1,
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mnl <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) 
apollo_modelOutput(mnl, modelOutput_settings)

##############################################################################################
####                                  Error component                                     ####
##############################################################################################

apollo_control = list(modelName  ="EC",modelDescr ="Error component model for TPN data",
                      indivID    ="ID", nCores = 4, outputDirectory = "output")
# SPECIFY COEFFICIENTS
apollo_beta = c(b_sq = 0.1, b_pass = 0.1, b_cost = 0.1, beta_ec = 0.5)
apollo_fixed = c()

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 200,
  interUnifDraws = c(),
  interNormDraws = c("draws_ec")
)
### Create random parameters
apollo_randCoeff = function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["EC"]] = beta_ec*draws_ec
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = b_pass*Passive_1+ b_cost*Cost_1 
  V[['alt2']]  = b_pass*Passive_2 + b_cost*Cost_2 
  V[['alt3']]  = b_sq + EC + b_pass*Passive_3 + b_cost*Cost_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = 1,
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Average across inter-individual draws
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
ec <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) # 
apollo_modelOutput(ec, modelOutput_settings)

##############################################################################################
####                                  Random parameter model                              ####
##############################################################################################

apollo_control = list(modelName  ="RP",modelDescr ="Random parameter model for TPN data",
                      indivID    ="ID", nCores = 4, outputDirectory = "output")
# SPECIFY COEFFICIENTS
apollo_beta = c(b_sq = 0.1, b_pass = 0.1, b_cost = 0.1, 
                s_sq = 0.1, s_pass = 0.1, s_cost = 0.1)
apollo_fixed = c()

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 200,
  interUnifDraws = c(),
  interNormDraws = c("draws_sq", "draws_pass", "draws_cost")
)
### Create random parameters
apollo_randCoeff = function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["SQ"]] = b_sq + s_sq*draws_sq
  randcoeff[["PASS"]] = b_pass + s_pass*draws_pass
  randcoeff[["COST"]] = b_cost + s_cost*draws_cost
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = PASS*Passive_1+ COST*Cost_1 
  V[['alt2']]  = PASS*Passive_2 + COST*Cost_2 
  V[['alt3']]  = SQ + PASS*Passive_3 + COST*Cost_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = 1,
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Average across inter-individual draws
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
rpl <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) 
apollo_modelOutput(rpl, modelOutput_settings)

## Calculate WTP
set.seed(1000001) 
draws<-rnorm(300000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- diag(rpl$estimate[seq(4,6)])
beta <- rpl$estimate[seq(1,3)] + VC %*% draws
WTP <- -beta[c(1,2),]/beta[3,]
colMeans(t(WTP))

set.seed(10000001) 
draws<-rnorm(300000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- diag(rpl$estimate[seq(4,6)])
beta <- rpl$estimate[seq(1,3)] + VC %*% draws
WTP <- -beta[c(1,2),]/beta[3,]
colMeans(t(WTP))

##############################################################################################
####                                  Random parameter model                              ####
####                                      Log-normal cost                                 ####
##############################################################################################

apollo_control = list(modelName  ="RP2",modelDescr ="Random parameter model for TPN data",
                      indivID    ="ID", nCores = 4, outputDirectory = "output")
# SPECIFY COEFFICIENTS
apollo_beta = c(b_sq = -0.1, b_pass = 0.25, b_cost = -0.3, 
                s_sq = 2, s_pass = 0.4, s_cost = 0.1)

apollo_fixed = c()

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 200,
  interUnifDraws = c(),
  interNormDraws = c("draws_sq", "draws_pass", "draws_cost")
)
### Create random parameters
apollo_randCoeff = function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["SQ"]] = b_sq + s_sq*draws_sq
  randcoeff[["PASS"]] = b_pass + s_pass*draws_pass
  randcoeff[["COST"]] = -exp(b_cost + s_cost*draws_cost)
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = PASS*Passive_1+ COST*Cost_1 
  V[['alt2']]  = PASS*Passive_2 + COST*Cost_2 
  V[['alt3']]  = SQ + PASS*Passive_3 + COST*Cost_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = 1,
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Average across inter-individual draws
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
rpl2 <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) 
apollo_modelOutput(rpl2, modelOutput_settings)
# Test
apollo_basTest(rpl,rpl2)


## Calculate WTP
set.seed(1000001) 
draws<-rnorm(300000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- abs(diag(rpl2$estimate[seq(4,6)]))
beta <- rpl2$estimate[seq(1,3)] + VC %*% draws
WTP <- beta[c(1,2),]/exp(beta[3,])
colMeans(t(WTP))

set.seed(100001) 
draws<-rnorm(300000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- abs(diag(rpl2$estimate[seq(4,6)]))
beta <- rpl2$estimate[seq(1,3)] + VC %*% draws
WTP <- beta[c(1,2),]/exp(beta[3,])
colMeans(t(WTP))

# Using median
set.seed(1000001) 
draws<-rnorm(30000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- abs(diag(rpl2$estimate[seq(4,6)]))
beta <- rpl2$estimate[seq(1,3)] + VC %*% draws
WTP <- beta[c(1,2),]/exp(beta[3,])
apply(WTP,1,median)

# Saving for starting values
apollo_saveOutput(rpl2, modelOutput_settings) # 

##############################################################################################
####                                  Random parameter model                              ####
####                                       Correlated RP                                  ####
##############################################################################################

apollo_control = list(modelName  ="RP_corr",modelDescr ="Random parameter model for TPN data",
                      indivID    ="ID", nCores = 4, outputDirectory = "output")
# SPECIFY COEFFICIENTS
apollo_beta = c(b_sq = -0.1, b_pass = 0.25, b_cost = -0.3, 
                s_sq = 2, 
                rho_pass_sq = 0.01, s_pass = 0.4, 
                rho_cost_sq = 0.01, rho_cost_pass = 0.01, s_cost = 0.1)
apollo_fixed = c()
apollo_beta=apollo_readBeta(apollo_beta,apollo_fixed,"RP2",overwriteFixed=FALSE)

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 200,
  interUnifDraws = c(),
  interNormDraws = c("draws_sq", "draws_pass", "draws_cost")
)

apollo_randCoeff = function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["SQ"]] = b_sq + s_sq*draws_sq
  randcoeff[["PASS"]] = b_pass + s_pass*draws_pass + rho_pass_sq*draws_sq
  randcoeff[["COST"]] = -exp(b_cost + s_cost*draws_cost + rho_cost_sq*draws_sq + rho_cost_pass*draws_pass)
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = PASS*Passive_1+ COST*Cost_1 
  V[['alt2']]  = PASS*Passive_2 + COST*Cost_2 
  V[['alt3']]  = SQ + PASS*Passive_3 + COST*Cost_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = 1,
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Average across inter-individual draws
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}
# Estimation                                      
rpl3 <- apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs) 
apollo_modelOutput(rpl3, modelOutput_settings)

apollo_lrTest(rpl2, rpl3)

## Calculate WTP
set.seed(1000001) 
draws<-rnorm(300000)
draws <- matrix(draws, nrow = 3, byrow = TRUE)
VC <- tril(ones(3, 3))
VC[VC==1] <- rpl3$estimate[c(4, 5, 7, 6, 8, 9)]
beta <- rpl3$estimate[seq(1,3)] + VC %*% draws
WTP <- beta[c(1,2),]/exp(beta[3,])
apply(WTP,1,median)


# Obtaining SE
VCOV<- rpl3$robvarcov
sqrt(diag(VCOV))
bhat <- rpl3$estimate
set.seed(1000001) 
BX <- rmvnorm(1000, mean = bhat, sigma = VCOV, method="chol")
WTP_sim <- matrix(, nrow = 1000, ncol = 2)

for(i in 1:1000){
  BX_i <- BX[i,] 
  draws<-rnorm(30000)
  draws <- matrix(draws, nrow = 3, byrow = TRUE)
  VC <- tril(ones(3, 3))
  VC[VC==1] <- BX_i[c(4, 5, 7, 6, 8, 9)]
  beta <- BX_i[seq(1,3)] + VC %*% draws
  WTP <- beta[c(1,2),]/exp(beta[3,])
  WTP_sim[i,] <- apply(WTP,1,median)
}

colMeans(WTP_sim)
apply(WTP_sim,2,sd)