library(plm)

####################  Simulation example no. 1 ########################
# Random effect model
 # 1000 respondents, 4 periods

rm(list=ls())
set.seed(1234567) # setting seed

data <- data.frame(matrix(ncol = 4, nrow = 4000))
colnames(data) <- c("id", "x1", "x2", "y")
data$x1 <- rnorm(4000,0,1)
data$x2 <- runif(4000,0,1)
re <- rnorm(1000,0,1) # random effect
data$id<- c(t(kronecker(matrix(1,1,4),1:1000)))
re2<-c(t(kronecker(matrix(1,1,4),re)))
eps <- rnorm(4000,0,2)

data$y <- 1 - 0.5*data$x1 + data$x2 + re2 + eps

summary(lm(y~ x1 + x2, data = data))
Panel_data<-pdata.frame(data,index = c("id"))

summary(plm(y~ x1 + x2,  model = "between", data = Panel_data))

summary(plm(y~ x1 + x2, data = Panel_data, model = "within"))
summary(plm(y~ x1 + x2, data = Panel_data, model = "random"))

m_f<-plm(y~ x1 + x2, data = Panel_data, model = "within")
m_r<-plm(y~ x1 + x2, data = Panel_data, model = "random")
phtest(m_f, m_r) # Hausman test 

# Fixed effect

rm(list=ls())
set.seed(1234567) # setting seed

data <- data.frame(matrix(ncol = 4, nrow = 4000))
colnames(data) <- c("id", "x1", "x2", "y")
data$x2 <- runif(4000,0,1)
fe <- rnorm(1000,0,1) # random effect
data$id<- c(t(kronecker(matrix(1,1,4),1:1000)))
fe2<-c(t(kronecker(matrix(1,1,4),fe)))
data$x1 <- (0.5*fe2 + rnorm(4000,0,1))/1.25

eps <- rnorm(4000,0,2)

data$y <- 1 - 0.5*data$x1 + data$x2 + fe2 + eps

summary(lm(y~ x1 + x2, data = data))
Panel_data<-pdata.frame(data,index = c("id"))

summary(plm(y~ x1 + x2,  model = "between", data = Panel_data))

summary(plm(y~ x1 + x2, data = Panel_data, model = "within"))
summary(plm(y~ x1 + x2, data = Panel_data, model = "random"))

m_f<-plm(y~ x1 + x2, data = Panel_data, model = "within")
m_r<-plm(y~ x1 + x2, data = Panel_data, model = "random")
phtest(m_f, m_r) # Hausman test 
